"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuiTransaction = void 0;
const utils_1 = require("ethers/lib/utils");
const EVMFunctionSignature_js_1 = require("../../businessObjects/EVMFunctionSignature.js");
const ChainTransaction_js_1 = require("../../businessObjects/versioned/ChainTransaction.js");
const index_js_1 = require("../../primitives/index.js");
/**
 * This is a concrete implementation of the Transaction class from Ethers. I'd really prefer to not have to
 * import and depend on Ethers directly, but odds are any project that is using @snickerdoodlelabs/objects
 * is also importing Ethers.
 * Docs are here: https://docs.ethers.io/v5/api/utils/transactions/#Transaction
 */
class SuiTransaction extends ChainTransaction_js_1.ChainTransaction {
    constructor(chain, hash, timestamp, blockHeight, to, from, value, gasPrice, contractAddress, input, methodId, functionName, events, measurementDate) {
        super(chain, hash, timestamp, measurementDate);
        this.chain = chain;
        this.hash = hash;
        this.timestamp = timestamp;
        this.blockHeight = blockHeight;
        this.to = to;
        this.from = from;
        this.value = value;
        this.gasPrice = gasPrice;
        this.contractAddress = contractAddress;
        this.input = input;
        this.methodId = methodId;
        this.functionName = functionName;
        this.measurementDate = measurementDate;
        this.functionSignature = null;
        let addrs = new Set();
        if (this.to) {
            addrs.add(this.to);
        }
        if (this.from) {
            addrs.add(this.from);
        }
        if (this.input &&
            this.functionName &&
            this.methodId &&
            !this.functionName.startsWith(this.methodId)) {
            try {
                const iface = new utils_1.Interface([`function ${this.functionName}`]);
                const func = iface.getFunction(this.input.slice(0, 10));
                const paramValues = iface.decodeFunctionData(func.name, this.input);
                // filter out unrecognized methodIDs
                // we may want to do a secondary lookup on failure here:
                // https://www.4byte.directory/docs/
                this.functionSignature = new EVMFunctionSignature_js_1.EVMFunctionSignature(func.name, func.type, func.inputs.map((input, i) => {
                    return new EVMFunctionSignature_js_1.EVMFunctionParameter(input.name, input.type, paramValues[i]);
                }));
                addrs = new Set([
                    ...addrs,
                    ...this._getDescendants(this.functionSignature),
                ]);
            }
            catch (e) {
                // this is also a bit noisy
                console.warn("error decoding transaction input", e);
            }
        }
        if (events) {
            addrs = new Set([...addrs, ...this._getDescendants(events)]);
        }
        this.accountAddresses = Array.from(addrs);
    }
    _getDescendants(obj) {
        let result = new Set();
        for (const [key, value] of Object.entries(obj)) {
            if (value && typeof value === "object") {
                result = new Set([...result, ...this._getDescendants(value)]);
            }
            else {
                if (typeof value === "string" && value.match(index_js_1.EVMAccountAddressRegex)) {
                    result.add((0, index_js_1.SuiAccountAddress)(value));
                }
            }
        }
        return result;
    }
}
exports.SuiTransaction = SuiTransaction;
//# sourceMappingURL=SuiTransaction.js.map