"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainErrorMapper = void 0;
const index_js_1 = require("../errors/index.js");
class BlockchainErrorMapper {
    static buildBlockchainError(error, generateGenericError) {
        // First check if error message is a ProviderError as some errors are triggered at the provider level rather than contract level
        try {
            let providerError;
            if ((error === null || error === void 0 ? void 0 : error.name) == "ProviderError") {
                providerError = this.getSpecificProviderError(error);
            }
            const errorReason = error === null || error === void 0 ? void 0 : error.reason;
            const errorMessage = (error === null || error === void 0 ? void 0 : error.message) || (error === null || error === void 0 ? void 0 : error.msg);
            const errorInitializerFromProviderError = this.blockchainErrorMapping.get(this.getErrorKey(providerError));
            const errorInitializerFromReason = this.blockchainErrorMapping.get(this.getErrorKey(errorReason));
            const errorInitializerFromMessage = this.blockchainErrorMapping.get(this.getErrorKey(errorMessage));
            if (errorInitializerFromProviderError != null) {
                return errorInitializerFromProviderError === null || errorInitializerFromProviderError === void 0 ? void 0 : errorInitializerFromProviderError(error);
            }
            if (errorInitializerFromReason != null) {
                return errorInitializerFromReason === null || errorInitializerFromReason === void 0 ? void 0 : errorInitializerFromReason(error);
            }
            if (errorInitializerFromMessage != null) {
                return errorInitializerFromMessage === null || errorInitializerFromMessage === void 0 ? void 0 : errorInitializerFromMessage(error);
            }
            // If all above are null, then we don't know what the error is
            if (generateGenericError != null) {
                return generateGenericError(errorMessage, errorReason, error);
            }
            return new index_js_1.UnknownBlockchainError(errorReason || errorMessage, error);
        }
        catch (e) {
            return new index_js_1.UnknownBlockchainError(`Error occurred while running buildBlockchainError ${e}`, e);
        }
    }
    static getErrorKey(errorString) {
        errorString = (errorString === null || errorString === void 0 ? void 0 : errorString.toLowerCase()) || "";
        Array.from(this.blockchainErrorMapping.keys()).forEach((key) => {
            if (errorString === null || errorString === void 0 ? void 0 : errorString.includes(key.toLowerCase())) {
                errorString = key;
            }
        });
        return errorString;
    }
    static getSpecificProviderError(error) {
        // ProviderError has the following properties : [ 'parent', 'name', '_stack', 'code', '_isProviderError', 'data' ]
        // The error message is within the '_stack' property
        // Search for a specific error text and return it if found,
        if (error._stack.includes("intrinsic gas too low")) {
            return "intrinsic gas too low";
        }
        // If no match is found, .includes() will return false, we do not recognize the type ProviderError and return null so that it can be captured as a generic error
        return null;
    }
}
exports.BlockchainErrorMapper = BlockchainErrorMapper;
BlockchainErrorMapper.blockchainErrorMapping = new Map([
    [
        "could not detect network",
        (error) => new index_js_1.NetworkUnreachableError("Could not detect network", error),
    ],
    [
        "insufficient funds for intrinsic transaction cost",
        (error) => new index_js_1.InsufficientFundsError("Insufficient funds to send transaction", error),
    ],
    [
        "sender doesn't have enough funds to send tx",
        (error) => new index_js_1.InsufficientFundsError("Insufficient funds to send transaction", error),
    ],
    [
        "resolver or addr is not configured for ENS name",
        (error) => new index_js_1.InvalidArgumentError("Address argument is invalid (format or type)", error),
    ],
    [
        "missing argument: passed to contract",
        (error) => new index_js_1.MissingArgumentError("Missing arguments for contract function call", error),
    ],
    [
        "too many arguments: passed to contract",
        (error) => new index_js_1.UnexpectedArgumentError("Too many arguments for contract function call", error),
    ],
    [
        "intrinsic gas too low",
        (error) => new index_js_1.GasTooLowError("Insufficient gas provided to function call", error),
    ],
    [
        "invalid address",
        (error) => new index_js_1.InvalidAddressError("Invalid address provided as argument to create contract instance", error),
    ],
    [
        "execution reverted",
        (error) => new index_js_1.ExecutionRevertedError("Function hits a revert message on the contract", error),
    ],
]);
//# sourceMappingURL=BlockchainErrorMapper.js.map